/*
 * Sistemas de Telecomunicações 
 *          2012/2013
 */
package protocol;

import simulator.Frame;
import terminal.NetworkLayer;

/**
 * Protocol 2 : Stop & Wait protocol
 * 
 * @author ?????
 */
public class Protocol2 implements Callbacks {

    public Protocol2(Simulator _sim, NetworkLayer _net) {
        this.sim = _sim;
        this.net = _net;
    }

    /**
     * Increment one unit a sequence number wrapping when the maximum sequence 
     * number is reached
     * @param n sequence number
     * @return the successor of the sequence number
     */
    final int incr_seq(int n) {
        return (n + 1) % (sim.get_max_sequence() + 1);
    }

    /**
     * Decrement one unit a sequence number wrapping when the maximum sequence 
     * number is reached
     * @param n sequence number
     * @return the predecessor of the sequence number
     */
    final int decr_seq(int n) {
        return (sim.get_max_sequence() + n) % (sim.get_max_sequence() + 1);
    }

    /**
     * CALLBACK FUNCTION: handle the beginning of the simulation event
     * @param time current simulation time
     */
    @Override
    public void start_simulation(long time) {
        sim.Log("\nStop&Wait Protocol\n\n");
        sim.Log("\nProtocol not implemented yet\n");
        sim.stop();
    }

    /**
     * CALLBACK FUNCTION: handle the end of Data frame transmission, start timer
     * @param time current simulation time
     * @param seq  sequence number of the Data frame transmitted
     */
    @Override
    public void handle_Data_end(long time, int seq) {
        // Not implemented yet
    }
    
    /**
     * CALLBACK FUNCTION: handle the timer event; retransmit failed frames
     * @param time current simulation time
     * @param key  timer key
     */
    @Override
    public void handle_Timer(long time, int key) {
        // Not implemented yet
    }
    
    /**
     * CALLBACK FUNCTION: handle the ack timer event; send ACK frame
     * @param time current simulation time
     */
    @Override
    public void handle_ack_Timer(long time) {
        // Not implemented yet
    }

    /**
     * CALLBACK FUNCTION: handle the reception of a frame from the physical layer
     * @param time current simulation time
     * @param frame frame received
     */
    @Override
    public void from_physical_layer(long time, Frame frame) {
        // Not implemented yet
    }

    /**
     * CALLBACK FUNCTION: handle the end of the simulation
     * @param time current simulation time
     */
    @Override
    public void end_simulation(long time) {
        sim.Log("Stopping simulation\n");
    }
    
    
    /* Variables */
    
    /**
     * Reference to the simulator (Terminal), to get the configuration and send commands
     */
    private Simulator sim;
    
    /**
     * Reference to the network layer, to send a receive packets
     */
    private NetworkLayer net;
    
}
