/*
 * Sistemas de Telecomunicações 
 *          2012/2013
 */
package protocol;

/**
 * Interface implemented by the Terminal that defines the methods the protocol 
 * can use to get the configuration parameters and send comands.
 * 
 * @author lflb@fct.unl.pt
 */
public interface Simulator extends simulator.Log { 
    
    /* Configuration parameters */
    
    /**
     * Get the sending window size
     * @return the sending window size
     */
    int get_send_window();
    
    /**
     * Get the receiving window size
     * @return the receiving window size
     */
    int get_recv_window();
    
    /**
     * Get the maximum sequence number
     * @return the maximum sequence number
     */
    int get_max_sequence();
    
    /**
     * Get the time interval the protocol should wait before retransmitting a data frame
     * @return the timeout value
     */
    long get_timeout();
    
    /**
     * Get to current simulation time
     * @return the current simulation time
     */
    long get_time();
    
    /*  Comands  */
    
    /**
     * Send one frame to the channel
     * @param frame frame to send
     */
    void to_physical_layer(simulator.Frame frame);
    
    /**
     * Start a timer for delay, associated to packet to a key; method
     * handle_timer of the protocol will be called after this time.
     * @param delay time interval
     * @param key a number equal to or above 0
     */
    void set_timer(long delay, int key);
    
    /**
     * Cancels the timer associated with the key
     * @param key timer key
     */
    void cancel_timer(int key);
    
    /**
     * Starts an ACK timer to wait for the transmission of a data frame before sending the ACK
     */
    void set_ack_timer();
        
    /**
     * Cancels the ACK timer
     */
    void cancel_ack_timer();
    
    /**
     * Stop the simulation
     */
    void stop();
    
    /**
     * Called by the protocol to count one extra data frame retransmission
     */
    void count_retransmission();
    
}
